<?php

class PatientController extends RestController
{

    /**
     * Defines which columns will be returned by the REST
     */
    public static $_columns = 'id,email,person_id,user_create_id,risk,forms,random_number,group_type,qathas';

    /**
     * If defined, specifies if the method is restricted to admin or not
     */
    public static $restricted = array(
        'post' => true,
        'delete' => true,
        'put' => true
    );

    /**
     * Do we cache this response?
     */
    protected $cacheable = false;

    /**
     * Allows search by and partial by fields
     */
    public static $_allow = array(
        'search' => array('id','email','person_id'),
        'partials' => array('id','email','person_id')
    );

    // @Override
    public function get($id = null){
        if ( ! $this->parseRequest(get_class($this)) ) {
            return ResponseHandler::badRequest($this);
        }

        $model_name = str_replace('Controller','',get_class($this));
        $controller_name = $model_name.'Controller';
        if (property_exists($controller_name,'restricted')){
            if (isset($controller_name::$restricted['get'])){
                if ($this->session->get('id') != $id && $this->session->get('level') != 1){
                    return ResponseHandler::forbidden($this);
                }
            }
        }

        if ( property_exists($model_name, 'valid') ){
            $delete_condition = ' AND valid=1';
        } else {
            $delete_condition = '';
        }

        if ($id){
            $object = $model_name::findFirst(array( "id =?1 ".$delete_condition,
                                                    "bind" => array( 1 => $id),
                                                    "columns" => $controller_name::$_columns),true);
        } else {
            
            if ($this->isSearch){
                $fieldsArray = "1=1 ";
                $field_id = 0;
                foreach ($this->searchFields as $field => $field_value){
                    $bindArray[$field_id] = $field_value;
                    $fieldsArray .= "AND ".$field."=?".$field_id; 
                    $field_id++;
                }
                $findArray = array(
                        $fieldsArray.$delete_condition,
                        "bind" => $bindArray
                );
            } else {
                $findArray = array(
                        "1=1 ".$delete_condition
                );
            }

            $findArray = $this->limit ? array_merge($findArray,array("limit" => $this->limit)):$findArray;
            $findArray = $this->offset ? array_merge($findArray,array("limit" => $this->offset)):$findArray;

            if ($this->isPartial){
                $findArray = array_merge($findArray, array("columns" => implode(',',$this->partialFields)));
            } else {
                $findArray = array_merge($findArray, array("columns" => $controller_name::$_columns));
            }

            $new_object = $model_name::find($findArray,!$this->isPartial);
            if ($new_object){
                foreach($new_object as $o){
                    $object2 = PatientForm::findFirst(array( "patient_id =?1 ",
                                                        "bind" => array( 1 => $o['id']),
                                                        "order" => "create_date desc",
                                                        "columns" => PatientFormController::$_columns),true);

                    
                    if ($object2){
                        $object2 = (array) $object2;
                        $object2['PatientHabits'] = PatientHabits::findFirst(array(
                            'patient_form_id = ?1',
                            'bind' => array( 1 => $object2['id'] )
                        ),true);
                        $object2['PatientExam'] = PatientExam::findFirst(array(
                            'patient_form_id = ?1',
                            'bind' => array( 1 => $object2['id'] )
                        ),true);
                        $object2['PatientDemographic'] = PatientDemographic::findFirst(array(
                            'patient_form_id = ?1',
                            'bind' => array( 1 => $object2['id'] )
                        ),true);
                        $object2['PatientClinical'] = PatientClinical::findFirst(array(
                            'patient_form_id = ?1',
                            'bind' => array( 1 => $object2['id'] )
                        ),true);
                        $object2['PatientComorbid'] = PatientComorbid::find(array(
                            'patient_form_id = ?1',
                            'bind' => array( 1 => $object2['id'] )
                        ),true);
                        $object2['PatientComplication'] = PatientComplication::find(array(
                            'patient_form_id = ?1',
                            'bind' => array( 1 => $object2['id'] )
                        ),true);
                        $object2['PatientDrug'] = PatientDrug::find(array(
                            'patient_form_id = ?1',
                            'bind' => array( 1 => $object2['id'] )
                        ),true);
                        $object2['PatientQuality'] = PatientQuality::findFirst(array(
                            'patient_form_id = ?1',
                            'bind' => array( 1 => $object2['id'] )
                        ),true);
                        $object2 = (object) $object2;
                    }
                    $o['Form'] = $object2;
                    $return_object[] = $o;
                }
                $object = $return_object;
            } else {
                $object = $new_object;
            }
            
            
        }
        return ResponseHandler::get($this,$object);
    }

    public function put($id = null){
        $patient = Patient::findFirst(array(
            'id = ?1',
            'bind' => array( 1 => $id )
        ), false);
        if ($patient){
            $request = $this->di->get('request')->getJsonRawBody();
            if ($request){
                $patient->email = $request->email;
                $patient->random_number = $request->random_number;
                $patient->group_type = $request->group_type;
                $patient->qathas = $request->qathas;
                $person = Person::findFirst(array(
                    'id = ?1',
                    'bind' => array( 1 => $patient->person_id )
                ), false);
                $person->name = $request->Person->name;
                $person->born_date = $request->Person->born_date;
                $person->phone = $request->Person->phone;
                $person->address = $request->Person->address;
                $person->neighborhood = $request->Person->neighborhood;
                $person->address_number = $request->Person->address_number;
                if ( $person->save() ) {
                    if ($patient->save()){
                        $updated_patient = Patient::findFirst(array( "id =?1",
                                                            "bind" => array( 1 => $patient->id),
                                                            "columns" => self::$_columns),true);
                        return ResponseHandler::put($this,$updated_patient,$patient->id);
                    } else {
                        foreach($patient->getMessages() as $m){
                            $err['error'][] = (string) $m;
                        }
                        return ResponseHandler::put($this,false,$patient->id,$err);
                    }
                } else {
                    foreach($person->getMessages() as $m){
                        $err['error'][] = (string) $m;
                    }
                    return ResponseHandler::put($this,false,$patient->id,$err);
                }
            } else {
                return ResponseHandler::put($this,false,$patient->id,array('error' => 'Sem dados para a requisição'));
            }
        } else {
            return ResponseHandler::put($this,false,false);
        }
    }

    public function get_form($id = null){
        if ($id){
            $object = PatientForm::find(array( "patient_id =?1 ",
                                                    "bind" => array( 1 => $id ),
                                                    "order" => "create_date desc"),false);
            $i = 0;
            foreach($object as $o){
                $array[$i]['id'] = $o->id;
                $array[$i]['number'] = $i;
                $array[$i]['create_date'] = $o->create_date;
                $array[$i]['points'] = $o->points;
                $i++;
            } 
        }
        return ResponseHandler::get($this,$array);
    }
}

