<?php

class PatientFormController extends RestController
{

    /**
     * Defines which columns will be returned by the REST
     */
    public static $_columns = 'id,patient_id,user_create_id,create_date,upload_date,points';

    /**
     * Do we cache this response?
     */
    protected $cacheable = false;

    /**
     * Allows search by and partial by fields
     */
    public static $_allow = array(
        'search' => array('id','patient_id','user_create_id','create_date','upload_date','points'),
        'partials' => array('id','patient_id','user_create_id','create_date','upload_date','points')
    );

    public function get($id = null){
        if ( ! $this->parseRequest(get_class($this)) ) {
            return ResponseHandler::badRequest($this);
        }

        if (property_exists('PatientFormController','restricted')){
            if (isset(PatientFormController::$restricted['get'])){
                if ($this->session->get('id') != $id && $this->session->get('level') != 1){
                    return ResponseHandler::forbidden($this);
                }
            }
        }
        if ($id){
            $object = PatientForm::findFirst(array( "id =?1 ",
                                                    "bind" => array( 1 => $id),
                                                    "columns" => PatientFormController::$_columns),true);
            if ($object){
                $object = (array) $object;
                $object['PatientHabits'] = PatientHabits::findFirst(array(
                    'patient_form_id = ?1',
                    'bind' => array( 1 => $object['id'] )
                ),true);
                $object['PatientExam'] = PatientExam::findFirst(array(
                    'patient_form_id = ?1',
                    'bind' => array( 1 => $object['id'] )
                ),true);
                $object['PatientDemographic'] = PatientDemographic::findFirst(array(
                    'patient_form_id = ?1',
                    'bind' => array( 1 => $object['id'] )
                ),true);
                $object['PatientClinical'] = PatientClinical::findFirst(array(
                    'patient_form_id = ?1',
                    'bind' => array( 1 => $object['id'] )
                ),true);
                $object['PatientComorbid'] = PatientComorbid::find(array(
                    'patient_form_id = ?1',
                    'bind' => array( 1 => $object['id'] )
                ),true);
                $object['PatientComplication'] = PatientComplication::find(array(
                    'patient_form_id = ?1',
                    'bind' => array( 1 => $object['id'] )
                ),true);
                $object['PatientDrug'] = PatientDrug::find(array(
                    'patient_form_id = ?1',
                    'bind' => array( 1 => $object['id'] )
                ),true);
                $object['PatientQuality'] = PatientQuality::findFirst(array(
                    'patient_form_id = ?1',
                    'bind' => array( 1 => $object['id'] )
                ),true);
                $object = (object) $object;
            }
        } else {
            if ($this->isSearch){
                $fieldsArray = "1=1 ";
                $field_id = 0;
                foreach ($this->searchFields as $field => $field_value){
                    $bindArray[$field_id] = $field_value;
                    $fieldsArray .= "AND ".$field."=?".$field_id; 
                    $field_id++;
                }
                $findArray = array(
                        $fieldsArray,
                        "bind" => $bindArray
                );
            } else {
                $findArray = array(
                        "1=1 "
                );
            }

            $findArray = $this->limit ? array_merge($findArray,array("limit" => $this->limit)):$findArray;
            $findArray = $this->offset ? array_merge($findArray,array("limit" => $this->offset)):$findArray;

            if ($this->isPartial){
                $findArray = array_merge($findArray, array("columns" => implode(',',$this->partialFields)));
            } else {
                $findArray = array_merge($findArray, array("columns" => PatientFormController::$_columns));
            }
            $findArray = array_merge($findArray, array('order' => 'create_date DESC'));

            $object = PatientForm::find($findArray,!$this->isPartial);

            if (is_object($object))
                $object = $object->toArray();
        }
        return ResponseHandler::get($this,$object);
    }

}

