<?php

use Phalcon\Mvc\Model\Behavior\SoftDelete;
use Phalcon\Validation\Validator\Email as Email;
use Phalcon\Validation\Validator\Uniqueness as Uniqueness;
use Phalcon\Validation\Validator\PresenceOf as PresenceOf;
use Phalcon\Validation\Validator\Between as Between;
use \Phalcon\Mvc\Model\Message as Message;
use Phalcon\Validation;

class Patient extends ModelBase
{
    /**
     * Handle nested results
     */
	public static $Relations = [
		'Person' => 'person_id',
        'User' => 'user_create_id'
	];

    /**
     * Clear cache for related models
     */
	// public static $clearCache = [
	// 	'ModelRelated'
	// ];

    public static $forbiddenFields = [
        'id','email','valid','forms'
    ];

    public $id;

    public $email;

    public $random_number;

    public $person_id;

    public $user_create_id;

    public $forms;

    public $risk;

    public $valid;

    public $group_type;

    public $qathas;

    public function validation(){

        $validator = new Validation();
        $translate = $this->di->get('translation');

        $validator->add(
            'email',
            new Email(
                array(
                    'required' => true,
                    'message' => $translate->t('validator.email')
                )
            )
        );
        
        $validator->add(
            'email',
            new Uniqueness(
                array(
                    "message" => $translate->t('unique.email')
                )
            )
        );

        $validator->add(
            'random_number',
            new Uniqueness(
                array(
                    "message" => $translate->t('unique.randomized_number')
                )
            )
        );

        $validator->add(
                'group_type',
                new Between(
                    array(
                        'minimum' => 1,
                        'maximum' => 2,
                        'message' => 'Grupo : '.$translate->t('validator.outofbounds')
                    )
                )
            );

        return $this->validate($validator);
    }

    /**
     * Initialize method for model.
     */
    public function initialize()
    {
        $this->belongsTo('person_id', 'Person', 'id', array('alias' => 'Person'));
        $this->belongsTo('user_create_id', 'User', 'id', array('alias' => 'UserCreate'));

        $this->hasMany('id', 'PatientForm', 'patient_id', array('alias' => 'PatientForm'));

        $this->addBehavior(
            new SoftDelete(
                array(
                    'field' => 'valid',
                    'value' => ModelConstants::DELETED
                )
            )
        );
    }

    /**
     * Returns table name mapped in the model.
     *
     * @return string
     */
    public function getSource()
    {
        return 'patient';
    }


    public function beforeValidationOnCreate(){
        $this->valid = 1;
        $this->forms = 0;
        $this->user_create_id = $this->getDI()->getSession()->get('id');
    }

}
