<?php

use Phalcon\Validation\Validator\PresenceOf as PresenceOf;
use Phalcon\Validation\Validator\Numericality as Numericality;
use Phalcon\Validation\Validator\Between as Between;
use \Phalcon\Mvc\Model\Message as Message;
use Phalcon\Validation;

class PatientExam extends ModelBase
{
    /**
     * Handle nested results
     */
    public static $Relations = [
        'PatientForm' => 'patient_form_id'
    ];

    /**
     * Clear cache for related models
     */
    // public static $clearCache = [
    //  'ModelRelated'
    // ];

    public static $forbiddenFields = [
        'patient_form_id'
    ];

    public $patient_form_id;

    public $weight;

    public $height;

    public $imc;

    public $imc_value;

    public $abs_circle;

    public $abs_circle_type;

    public $blood_pressure_systole;

    public $blood_pressure_diastole;

    public $blood_pressure_type;

    public $itc;

    public $itc_value;

    public $diagnosis;

    public $prescription;

    public $evolution;

    public $history;

    public $exam;

    public function validation(){

        $validator = new Validation();
        $translate = $this->di->get('translation');

        $validator->add(
                'imc',
                new Between(
                    array(
                        'minimum' => 1,
                        'maximum' => 4,
                        'message' => 'IMC : ' . $translate->t('validator.outofbounds')
                    )
                )
            );

        $validator->add(
                'abs_circle_type',
                new Between(
                    array(
                        'minimum' => 1,
                        'maximum' => 2,
                        'message' => 'Circunferência Abdominal : ' . $translate->t('validator.outofbounds')
                    )
                )
            );

        $validator->add(
                'blood_pressure_type',
                new Between(
                    array(
                        'minimum' => 1,
                        'maximum' => 6,
                        'message' => 'Pressão Arterial : ' . $translate->t('validator.outofbounds')
                    )
                )
            );

        $validator->add(
                'itc',
                new Between(
                    array(
                        'minimum' => 1,
                        'maximum' => 4,
                        'message' => 'Índice Tornozelo-Braquial : ' . $translate->t('validator.outofbounds')
                    )
                )
            );

        $validator->add(
                ['height','weight'],
                new Numericality(
                    array(
                        'message' => [
                            'height' => 'Altura : '.$translate->t('validator.number'),
                            'weight' => 'Peso : '.$translate->t('validator.number')
                        ]
                    )
                )
            );
        //@TODO Implementar validações
        return $this->validate($validator);
    }

    /**
     * Initialize method for model.
     */
    public function initialize()
    {
        $this->belongsTo('patient_form_id', 'PatientForm', 'id', array('alias' => 'PatientForm'));
    }

    /**
     * Returns table name mapped in the model.
     *
     * @return string
     */
    public function getSource()
    {
        return 'patient_exam';
    }

}
