<?php

use Phalcon\Validation\Validator\PresenceOf as PresenceOf;
use \Phalcon\Mvc\Model\Message as Message;
use Phalcon\Validation;

class PatientForm extends ModelBase
{
    /**
     * Handle nested results
     */
	public static $Relations = [
		'Patient' => 'patient_id',
        'User' => 'user_create_id'
	];

    /**
     * Clear cache for related models
     */
	// public static $clearCache = [
	// 	'ModelRelated'
	// ];

    public static $forbiddenFields = [
        'id'
    ];

    public $id;

    public $patient_id;

    public $user_create_id;

    public $create_date;

    public $upload_date;

    public $points;

    public function validation(){

        $validator = new Validation();
        $translate = $this->di->get('translation');

        $validator->add(
            'patient_id',
            new PresenceOf(
                array(
                    'message' => $translate->t('validator.patient_id')
                )
            )
        );
        
        $validator->add(
            'create_date',
            new PresenceOf(
                array(
                    'message' => $translate->t('validator.create_date')
                )
            )
        );

        return $this->validate($validator);
    }

    /**
     * Initialize method for model.
     */
    public function initialize()
    {
        $this->belongsTo('patient_id', 'Patient', 'id', array('alias' => 'Patient'));
        $this->belongsTo('user_create_id', 'User', 'id', array('alias' => 'UserCreate'));

        $this->hasOne('id', 'PatientExam', 'patient_form_id', array('alias' => 'PatientExam'));
        $this->hasOne('id', 'PatientHabits', 'patient_form_id', array('alias' => 'PatientHabits'));
        $this->hasOne('id', 'PatientDemographic', 'patient_form_id', array('alias' => 'PatientDemographic'));
        $this->hasOne('id', 'PatientClinical', 'patient_form_id', array('alias' => 'PatientClinical'));

        $this->hasMany('id', 'PatientDrug', 'patient_form_id', array('alias' => 'PatientDrug'));
        $this->hasMany('id', 'PatientComorbid', 'patient_form_id', array('alias' => 'PatientComorbid'));
        $this->hasMany('id', 'PatientComplication', 'patient_form_id', array('alias' => 'PatientComplication'));

        $this->hasOne('id', 'PatientQuality', 'patient_form_id', array('alias' => 'PatientQuality'));
    }

    /**
     * Returns table name mapped in the model.
     *
     * @return string
     */
    public function getSource()
    {
        return 'patient_form';
    }

    public function beforeDelete(){
        $patient = Patient::findFirst([
            'id = ?1',
            'bind' => array( 1 => $this->patient_id )
        ],false);
        $patient->forms = $patient->forms - 1;
        $patient->save();
    }

    public function afterCreate(){
        $patient = Patient::findFirst([
            'id = ?1',
            'bind' => array( 1 => $this->patient_id )
        ],false);
        $patient->forms = $patient->forms + 1;
        $patient->risk = $this->points;
        $patient->save();

        $patientExam = PatientExam::findFirst([
            'patient_form_id = ?1',
            'bind' => [ 1 => $this->id ]
        ],false);

        $email_plugin = new MailPlugin();
        $data = ['FIRSTNAME' => $patient->Person->name,
                'DATAEXAME' => date('d/m/Y H:i',$this->create_date),
                'NOMEENFERMEIRO' => $this->UserCreate->Person->name,
                'PRESCRIPTION' => $patientExam->prescription];
                
        $email_plugin->send($patient->email,MailConstants::FORM,$data);

    }

    public function beforeValidationOnCreate(){
        $this->upload_date = time();
        $this->user_create_id = $this->getDI()->getSession()->get('id');

        $request = $this->di->get('request')->getJsonRawBody();
        $translate = $this->di->get('translation');
        try { 
            if (!isset($request->PatientHabits))
                throw new Exception('notfound.habits');
            if (!isset($request->PatientExam))
                throw new Exception('notfound.exam');
            if (!isset($request->PatientDemographic))
                throw new Exception('notfound.demographic');
            if (!isset($request->PatientClinical))
                throw new Exception('notfound.clinical');
            if (!isset($request->PatientQuality))
                throw new Exception('notfound.quality');
        } catch (Exception $e){
            $this->appendMessage(new Message($translate->t($e->getMessage())));
            return false;
        }
        $this->points = 0;
        $this->points += ($request->PatientDemographic->age > 55) ? 10 : 0;
        $this->points += ($request->PatientDemographic->sex == 1) ? 12 : 0;
        $this->points += ($request->PatientHabits->smoke == 1) ? 11 : 0;
        $this->points += ($request->PatientClinical->hypertension == 4) ? 13 : 0;
        $this->points += ($request->PatientClinical->drugs_in_use > 3) ? 22 : 0;
        $this->points += ($request->PatientClinical->depression == 1) ? 14 : 0;
        $this->points += ($request->PatientClinical->risk > 1) ? 18 : 0;

    }

}
