<?php

use Phalcon\Validation\Validator\Uniqueness as Uniqueness;
use Phalcon\Validation\Validator\PresenceOf as PresenceOf;
use Phalcon\Validation\Validator\StringLength as StringLength;
use \Phalcon\Mvc\Model\Message as Message;
use Phalcon\Validation;
use Phalcon\Filter;

class Person extends ModelBase
{
    /**
     * Handle nested results
     */
	// public static $Relations = [
	// 	'ModelChild' => 'foreign_key_id'
	// ];

    /**
     * Clear cache for related models
     */
	// public static $clearCache = [
	// 	'ModelRelated'
	// ];

    public static $forbiddenFields = [
        'id'
    ];

    public $id;

    public $name;

    public $born_date;

    public $phone;

    public $address;

    public $neighborhood;

    public $address_number;

    public $photo;

    public function validation(){

        $validator = new Validation();
        $translate = $this->di->get('translation');

        $validator->add(
            'name',
            new PresenceOf(
                array(
                    'message' => $translate->t('validator.name')
                )
            )
        );

        $validator->add(
            'name',
            new StringLength(
                array(
                    'max' => 200,
                    'min' => 5,
                    'messageMaximum' => $translate->t('validator.max.name'),
                    'messageMinimum' => $translate->t('validator.min.name')
                )
            )
        );

        $validator->add(
            'born_date',
            new PresenceOf(
                array(
                    'message' => $translate->t('validator.borndate')
                )
            )
        );

        $validator->add(
            'phone',
            new PresenceOf(
                array(
                    'message' => $translate->t('validator.phone')
                )
            )
        );

        $validator->add(
            'phone',
            new StringLength(
                array(
                    'max' => 11,
                    'min' => 10,
                    'messageMaximum' => $translate->t('validator.max.phone'),
                    'messageMinimum' => $translate->t('validator.min.phone')
                )
            )
        );

        return $this->validate($validator);
    }

    /**
     * Initialize method for model.
     */
    public function initialize()
    {
        $this->hasOne('id', 'User', 'person_id', array('alias' => 'User'));
        $this->hasOne('id', 'Patient', 'person_id', array('alias' => 'Patient'));

    }

    /**
     * Returns table name mapped in the model.
     *
     * @return string
     */
    public function getSource()
    {
        return 'person';
    }

    public function beforeValidation(){
        $this->name = Phalcon\Text::camelize($this->name);
        $this->address = Phalcon\Text::camelize($this->address);
        $this->neighborhood = Phalcon\Text::camelize($this->neighborhood);
        $filter = new Filter();
        $this->born_date = $filter->sanitize($this->born_date,'int');
        $this->phone = $filter->sanitize(str_replace(array('-','+'),'',$this->phone),'int');
        $this->address_number = $filter->sanitize($this->address_number,'int');
    }


}
