<?php

use Phalcon\Mvc\Model\Behavior\SoftDelete;
use Phalcon\Validation\Validator\Email as Email;
use Phalcon\Validation\Validator\Uniqueness as Uniqueness;
use Phalcon\Validation\Validator\PresenceOf as PresenceOf;
use \Phalcon\Mvc\Model\Message as Message;
use Phalcon\Validation;

class User extends ModelBase
{
    /**
     * Handle nested results
     */
	public static $Relations = [
		'Person' => 'person_id'
	];

    /**
     * Clear cache for related models
     */
	// public static $clearCache = [
	// 	'ModelRelated'
	// ];

    public static $forbiddenFields = [
        'id','email','valid'
    ];

    public $skipRequest = false;

    public $id;

    public $email;

    public $password;

    public $person_id;

    public $valid;

    public function validation(){

        $validator = new Validation();
        $translate = $this->di->get('translation');

        $validator->add(
            'email',
            new Email(
                array(
                    'required' => true,
                    'message' => $translate->t('validator.email')
                )
            )
        );
        
        $validator->add(
            'email',
            new Uniqueness(
                array(
                    "message" => $translate->t('unique.email')
                )
            )
        );

        return $this->validate($validator);
    }

    /**
     * Initialize method for model.
     */
    public function initialize()
    {
        $this->hasMany('id', 'Log', 'user_id', array('alias' => 'Log'));
        $this->hasMany('id', 'Apikey', 'user_id', array('alias' => 'Apikey'));
        $this->hasMany('id', 'PasswordRecovery', 'user_id', array('alias' => 'PasswordRecovery'));
        $this->hasMany('id', 'Patient', 'create_user_id', array('alias' => 'PatientCreator'));
        $this->belongsTo('person_id', 'Person', 'id', array('alias' => 'Person'));

        $this->addBehavior(
            new SoftDelete(
                array(
                    'field' => 'valid',
                    'value' => ModelConstants::DELETED
                )
            )
        );
    }

    /**
     * Returns table name mapped in the model.
     *
     * @return string
     */
    public function getSource()
    {
        return 'user';
    }


    public function beforeValidationOnCreate(){
        $this->valid = 1;
        $this->password = sha1($this->password);
    }

    public function beforeValidationOnUpdate(){
        if ($this->skipRequest){
            return true;
        } else {
            $request = $this->di->get('request')->getJsonRawBody();
            $translate = $this->di->get('translation');
            if ( $this->_comparePassword($request,$translate) ) {
                return $this->validateRequest($request,$translate);
            } else{
                return true;
            }
        }
    }

    public function beforeUpdate(){
        if ($this->skipRequest)
            return true;
        $request = $this->di->get('request')->getJsonRawBody();
        if (isset($request)){
            if ($request->new_password){
                $this->password = sha1($request->new_password);
            }
        }
        return true;
    }
    private function _comparePassword($request,$translate){
        if (isset($request)){
            if ( isset($request->old_password) ){
                if ($this->password != sha1($request->old_password)){
                    $this->appendMessage(new Message($translate->t('error.oldpassword')));
                    return false;
                }
            } else {
                $this->appendMessage(new Message($translate->t('notfound.oldpassword')));
                return false;
            }
        }
        return true;
    }

}
