<?php
use Phalcon\Acl;
use Phalcon\Acl\Role;
use Phalcon\Acl\Resource;
use Phalcon\Events\Event;
use Phalcon\Mvc\User\Plugin;
use Phalcon\Mvc\Dispatcher;
use Phalcon\Acl\Adapter\Memory as AclList;
/**
 * SecurityPlugin
 *
 * This is the security plugin which controls that users only have access to the modules they're assigned to
 */
class AclPlugin extends Plugin
{
	/**
	 * Returns an existing or new access control list
	 *
	 * @returns AclList
	 */
	public function getAcl()
	{
		// if (!isset($this->persistent->acl)) {
			$acl = new AclList();
			$acl->setDefaultAction(Acl::DENY);
			//Register roles
			$roles = array(
				'public'  => new Role('Public'),
				'cliente' => new Role('Client'),
				'administrador' => new Role('Admin')
			);
			foreach ($roles as $role) {
				$acl->addRole($role);
			}
			$acl->addInherit('Client','Public');
			$acl->addInherit('Admin','Client');


			$resources = [
			    'Public'=>[
			    	"PasswordRecovery" => ["post","put","get"],
			    	"Cid10" => ["get"],
			    	"Drug" => ["get"],
				"Qathas" => ["post"]
			    ],
			    'Client'=>[
			    	"User"=>["put","get","post","delete"],
			    	"Patient"=>["put","get","post","delete","get_form"],
			    	"PatientForm" => ['get','post','put','delete'],
			    	"Drug" => ['delete','post',"put"],
			    	"Person"=>["get"],
			    	"Login" => ["get"],
			    	"Dump" => ["export"],
			    	"Statistics" => ["get"]
			    ],
			    'Admin'=>[
			    	"User" => ["delete"]
			    ]
			];
			foreach($resources as $resource){
			    foreach($resource as $controller=>$arrMethods){
			        $acl->addResource(new Phalcon\Acl\Resource($controller . 'Controller'),$arrMethods);
			    }
			}

			/*
			 * ACCESS
			 * */
			foreach ($acl->getRoles() as $objRole) {
			    $roleName = $objRole->getName();
			    //publico
			    foreach ($resources['Public'] as $resource => $method) {
			        $acl->allow($roleName,$resource . 'Controller',$method);
			    }

			    if($roleName == 'Client'){
			        foreach ($resources['Client'] as $resource => $method) {
			            $acl->allow($roleName,$resource . 'Controller',$method);
			        }
			    }

			    //admins
			    if($roleName == 'Admin'){
			        foreach ($resources['Admin'] as $resource => $method) {
			            $acl->allow($roleName,$resource . 'Controller',$method);
			        }
			    }
			}//The acl is stored in session, APC would be useful here too
			$this->persistent->acl = $acl;
		// }
		return $this->persistent->acl;
	}

	public function getPermission($app){
		$acl = $this->getAcl();
		$arrHandler = $app->getActiveHandler();
		$controller = str_replace('Controller\\','',get_class($arrHandler[0]));
		switch($app->session->get('level')){
			case 2: // Admin
				return $acl->isAllowed('Admin', $controller, $arrHandler[1]);
			break;
			case 1: // Usuario
				return $acl->isAllowed('Client', $controller, $arrHandler[1]);
			break;
			default : //Public
				return $acl->isAllowed('Public', $controller, $arrHandler[1]);
			break;
		}
	}
	
}

?>
