<?php

require __DIR__ . '/amazon/vendor/autoload.php';
use Aws\S3\S3Client;
use Aws\S3\Exception\S3Exception;

class FilePlugin
{

	private $_bucket = 'caixinhadecuidados';

	private $_client;

	public function __construct($bucket = null,$key,$secret){
		$this->_client = S3Client::factory(array(
			'credentials' => array(
				'key' => $key,
				'secret' => $secret
			)
		));
		if ($bucket)
			$this->_bucket = $bucket;
	}


	private static function doFileName($source,$extension = null){
		return date('Y/m/d/').time().'_'.rand(0,100).$extension;
	}

	public function upload($source,$private = false,$key = null,$sync = null){
		try {
			if (!$key)
				$key = self::doFileName($source);

			if (!$private)
				$acl = 'public-read';
			else
				$acl = 'private';

			$result = $this->_client->putObject(array(
				'Bucket' => $this->_bucket,
				'Key' => $key,
				'SourceFile' => $source,
				'ACL' => $acl
			));

			if ($sync){
				$this->_client->waitUntil('ObjectExists',array(
					'Bucket' => $this->_bucket,
					'Key' => $key
				));
			}
			$result['key'] = $key;
			return $result;
		} catch (S3Exception $e){
			return false;
		}
	}

	public function fromBase64($base64){
		$data = explode(',',$base64);
		if (!isset($data))
			return null;
		$filetype = explode(';',$data[0]);
		$filetype = explode('/',$filetype[0]);
		if (!isset($filetype[1])){
			$filetype = '.jpg';
		} else {
			$filetype = $filetype[1];
		}
		$filename = '/tmp/'.time().'_'.rand(0,100).'.'.$filetype;
		$fp = fopen($filename,'wb');
		fwrite($fp,base64_decode($data[1]));
		fclose($fp);
		return $filename;
	}

}

